#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (C) 2008,2015 Sergey Poznyakoff
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from WikiTrans.wikitoken import *
from WikiTrans.wikimarkup import *
from WikiTrans.wikins import wiki_ns_re, wiki_ns
import re
try:
    from urllib import quote as url_quote
except ImportError:
    from urllib.parse import quote as url_quote

class TextSeqNode(WikiSeqNode):
    def format(self):
        string = ""
        for x in self.content:
            if len(string) > 1 and not string[-1].isspace():
                string += ' '
            string += x.format()
        return string
    
class TextTextNode(WikiTextNode):
    def format(self):
        if isinstance(self.content,list):
            string = ""
            for s in self.content:
                if string:
                    if string.endswith("."):
                        string += "  "
                    else:
                        string += " "
                string += s
        else:
            string = self.content
        return string

class TextPreNode(WikiSeqNode):
    def format(self):
        string = ""
        for x in self.content:
            string += x.format()
        string += '\n'
        return string

class TextParaNode(WikiSeqNode):
    def format(self):
        string = ""
        for x in self.content:
            string += x.format()
        string = self.parser.fmtpara(string) + '\n\n'
        return string

class TextItNode(WikiSeqNode):
    def format(self):
        string = ""
        for x in self.content:
            s = x.format()
            if s:
                string += " " + s
        return "_" + string.lstrip(" ") + "_"
        
class TextBoldNode(WikiSeqNode):
    def format(self):
        string = ""
        for x in self.content:
            if string.endswith("."):
                string += "  "
            else:
                string += " "
            string += x.format()
        return string.upper()

class TextLinkNode(WikiSeqNode):
    def format(self):
        arg = self.content[0].format()
        if len(self.content) > 1:
            s = [x for x in map(lambda x: x.format(), self.content)]
            text = s[1]
        else:
            s = None
            text = None

        if s:
            if s[0] == 'disambigR' or s[0] == 'wikiquote':
                return ""
            if len(s) > 1 and s[1] == 'thumb':
                return ""
        (qual,sep,tgt) = arg.partition(':')
        if tgt != '':
            ns = self.parser.wiki_ns_name(qual)
            if ns:
                if ns == 'NS_IMAGE':
                    if not self.parser.references:
                        return ""
                    text = "[%s: %s]" % (qual, text if text else arg)
                    tgt = self.image_base + '/' + \
                                 url_quote(tgt) + \
                                 '/250px-' + url_quote(tgt)
                elif ns == 'NS_MEDIA':
                    text = "[%s]" % (qual)
                else:
                    tgt = self.parser.mktgt(tgt)
            elif self.type == 'LINK' and qual in self.parser.langtab:
                text = self.parser.langtab[qual] + ": " + tgt
                tgt = self.parser.mktgt(tgt, qual)
            else:
                tgt = self.parser.mktgt(tgt)
        else:
            tgt = self.parser.mktgt(arg)
        if self.parser.references:
            return "%s (see %s) " % (text, tgt)
        elif not text or text == '':
            return arg
        else:
            return text
        
class TextTmplNode(TextLinkNode):
    def format(self):
        return '[' + super(TextTmplNode, self).format() + ']'
        
class TextBarNode(WikiNode):
    def format(self):
        w = self.parser.width
        if w < 5:
            w = 5
        return "\n" + ("-" * (w - 5)).center(w - 1) + "\n"

class TextHdrNode(WikiHdrNode):
    def format(self):
        return "\n" + ("*" * self.level) + " " + \
                      self.content.format().lstrip(" ") + "\n\n"
        
class TextRefNode(WikiRefNode):
    def format(self):
        text = self.content.format()
        if text:
            return "%s (see %s) " % (text, self.ref)
        else:
            return "see " + self.ref

class TextEnvNode(WikiEnvNode):
    def format(self):
        type = self.envtype
        lev = self.level
        if lev > self.parser.width - 4:
            lev = 1
        string = ""
        n = 1
        for s in self.content:
            if not string.endswith("\n"):
                string += "\n"
            x = s.content.format()
            if type == "unnumbered":
                string += self.parser.indent(lev, "- " + x.lstrip(" "))
            elif type == "numbered":
                string += self.parser.indent(lev, "%d. %s" % (n, x))
                n += 1
            elif type == "defn":
                if s.subtype == 0:
                    string += self.parser.indent(lev-1, x)
                else:
                    string += self.parser.indent(lev+3, x)

            if not string.endswith("\n"):
                string += "\n"

        return string

class TextIndNode(WikiIndNode):
    def format(self):
        return (" " * self.level) + self.content.format() + '\n'

class TextTagNode(WikiTagNode):
    def format(self):
        if self.tag == 'code':
            self.parser.nested += 1
            s = self.content.format()
            self.parser.nested -= 1
        else:
            s = '<' + self.tag
            if self.args:
                s += ' ' + str(self.args)
            s += '>' + self.content.format() + '</' + self.tag + '>'
        return s            
    

class TextWikiMarkup (WikiMarkup):
    """
    A (general-purpose Wiki->Text translator class.
    """

    # Output width
    width = 78
    # Do not show references.
    references = False
    # Provide a minimum markup
    markup = True

    # Number of current element in the environment
    num = 0
    
    def __init__(self, *args, **keywords):
        super(TextWikiMarkup,self).__init__(*args, **keywords)
        if 'width' in keywords:
            self.width = keywords['width']
        if 'refs' in keywords:
            self.references = keywords['refs']
        if 'markup' in keywords:
            self.markup = keywords['markup']
        self.token_class['SEQ'] = TextSeqNode
        self.token_class['TEXT'] = TextTextNode
        self.token_class['PRE'] = TextPreNode
        self.token_class['PARA'] = TextParaNode
        self.token_class['SEQ'] = TextSeqNode
        self.token_class['IT'] = TextItNode
        self.token_class['BOLD'] = TextBoldNode
        self.token_class['LINK'] = TextLinkNode
        self.token_class['TMPL'] = TextTmplNode
        self.token_class['BAR'] = TextBarNode
        self.token_class['HDR'] = TextHdrNode
        self.token_class['REF'] = TextRefNode
        self.token_class['ENV'] = TextEnvNode
        self.token_class['IND'] = TextIndNode
        self.token_class['TAG'] = TextTagNode
    
    def wiki_ns_name(self, str):
        if str in wiki_ns[self.lang]:
            return wiki_ns[self.lang][str]
        elif str in wiki_ns_re[self.lang]:
            for elt in wiki_ns_re[self.lang][str]:
                if str.beginswith(elt[0]) and str.endswith(elt[1]):
                    return elt[2]
        return None
    
    def mktgt(self, tgt, lang = None):
        if not lang:
            lang = self.lang
        return self.html_base % { 'lang' : lang } + url_quote(tgt)
    
    def fmtlink(self, elt, istmpl):
        arg = self.format(elt.content[0])
        if len(elt.content) > 1:
            s = [x for x in map(self.format, elt.content)]
            text = s[1]
        else:
            s = None
            text = None

        if s:
            if s[0] == 'disambigR' or s[0] == 'wikiquote':
                return ""
            if len(s) > 1 and s[1] == 'thumb':
                return ""
        (qual,sep,tgt) = arg.partition(':')
        if tgt != '':
            ns = self.wiki_ns_name(qual)
            if ns:
                if ns == 'NS_IMAGE':
                    if not self.references:
                        return ""
                    text = "[%s: %s]" % (qual, text if text else arg)
                    tgt = self.image_base + '/' + \
                                 url_quote(tgt) + \
                                 '/250px-' + url_quote(tgt)
                elif ns == 'NS_MEDIA':
                    text = "[%s]" % (qual)
                else:
                    tgt = self.mktgt(tgt)
            elif not istmpl and qual in self.langtab:
                text = self.langtab[qual] + ": " + tgt
                tgt = self.mktgt(tgt, qual)
            else:
                tgt = self.mktgt(tgt)
        else:
            tgt = self.mktgt(arg)
        if self.references:
            return "%s (see %s) " % (text, tgt)
        elif not text or text == '':
            return arg
        else:
            return text

    def indent (self, lev, text):
        if text.find('\n') == -1:
            s = (" " * lev) + text 
        else:
            s = ""
            for elt in text.split('\n'):
                if elt:
                    s += (" " * lev) + elt + '\n'
            if not text.endswith('\n'):
                s = s.rstrip('\n')
#        print "IN: '%s'" % (text)
#        print "OUT: '%s'" % (s)
        return s
    
    def fmtpara(self, input):
        output = ""
        linebuf = ""
        length = 0
        for s in input.split():
            wlen = len(s)
            if len(linebuf) == 0:
                wsc = 0
            elif linebuf.endswith("."):
                wsc = 2
            else:
                wsc = 1
            if length + wsc + wlen > self.width:
                # FIXME: fill out linebuf
                output += linebuf + '\n'
                wsc = 0
                length = 0
                linebuf = ""
            linebuf += " " * wsc + s
            length += wsc + wlen
        return output + linebuf
        
    def str_tag(self, elt):
        if elt.tag == 'code':
            self.nested += 1
            s = self.format(elt.content)
            self.nested -= 1
            return s #FIXME
        else:
            s = '<' + elt.tag
            if elt.args:
                s += ' ' + str(elt.args)
            s += '>' + self.format(elt.content) + '</' + elt.tag + '>'
            return s            
        
    def __str__(self):
        str = ""
        for elt in self.tree:
            str += elt.format()
        return str

class TextWiktionaryMarkup (TextWikiMarkup):
    """
 See documentation for HtmlWiktionaryMarkup
    """
    # FIXME: It is supposed to do something about templates

